/* GTK - The GIMP Toolkit
 * eggpagesettings.c: Page Settings
 * Copyright (C) 2005, Red Hat, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "eggpagesettings.h"

typedef struct _EggPageSettingsClass EggPageSettingsClass;

#define EGG_IS_PAGE_SETTINGS_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), EGG_TYPE_PAGE_SETTINGS))
#define EGG_PAGE_SETTINGS_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), EGG_TYPE_PAGE_SETTINGS, EggPageSettingsClass))
#define EGG_PAGE_SETTINGS_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), EGG_TYPE_PAGE_SETTINGS, EggPageSettingsClass))

struct _EggPageSettings
{
  GObject parent_instance;

  EggPageOrientation orientation;
  EggPaperSize *paper_size;
  double top_margin, bottom_margin, left_margin, right_margin;
};

struct _EggPageSettingsClass
{
  GObjectClass parent_class;
};

G_DEFINE_TYPE (EggPageSettings, egg_page_settings, G_TYPE_OBJECT)

static void
egg_page_settings_finalize (GObject *object)
{
  EggPageSettings *settings = EGG_PAGE_SETTINGS (object);
  
  g_object_unref (settings->paper_size);
  
  G_OBJECT_CLASS (egg_page_settings_parent_class)->finalize (object);
}

static void
egg_page_settings_init (EggPageSettings *settings)
{
  settings->orientation = EGG_PAGE_ORIENTATION_PORTRAIT;
  settings->paper_size = egg_paper_size_new (EEL_PAPER_NAME_A4);
  settings->top_margin = 10;
  settings->bottom_margin = 10;
  settings->left_margin = 10;
  settings->right_margin = 10;
}

static void
egg_page_settings_class_init (EggPageSettingsClass *class)
{
  GObjectClass *gobject_class = (GObjectClass *)class;

  gobject_class->finalize = egg_page_settings_finalize;
}

  
EggPageSettings *
egg_page_settings_new (void)
{
  return g_object_new (EGG_TYPE_PAGE_SETTINGS, NULL);
}

EggPageSettings *
egg_page_settings_copy (EggPageSettings *other)
{
  EggPageSettings *copy;

  copy = egg_page_settings_new ();
  copy->orientation = other->orientation;
  copy->paper_size = egg_paper_size_copy (other->paper_size);
  copy->top_margin = other->top_margin;
  copy->bottom_margin = other->bottom_margin;
  copy->left_margin = other->left_margin;
  copy->right_margin = other->right_margin;

  return copy;
}

EggPageOrientation
egg_page_settings_get_orientation (EggPageSettings *settings)
{
  return settings->orientation;
}

void
egg_page_settings_set_orientation (EggPageSettings    *settings,
				   EggPageOrientation  orientation)
{
  settings->orientation = orientation;
}

EggPaperSize *
egg_page_settings_get_paper_size (EggPageSettings    *settings)
{
  return settings->paper_size;
}

void
egg_page_settings_set_paper_size (EggPageSettings    *settings,
				  EggPaperSize        *size)
{
  settings->paper_size = g_object_ref (size);
}

double
egg_page_settings_get_top_margin (EggPageSettings    *settings)
{
  return settings->top_margin;
}

void
egg_page_settings_set_top_margin (EggPageSettings    *settings,
				  double              margin)
{
  settings->top_margin = margin;
}

double
egg_page_settings_get_bottom_margin (EggPageSettings *settings)
{
  return settings->bottom_margin;
}

void
egg_page_settings_set_bottom_margin (EggPageSettings    *settings,
				     double              margin)
{
  settings->bottom_margin = margin;
}

double
egg_page_settings_get_left_margin (EggPageSettings    *settings)
{
  return settings->left_margin;
}

void
egg_page_settings_set_left_margin (EggPageSettings    *settings,
				   double              margin)
{
  settings->left_margin = margin;
}

double
egg_page_settings_get_right_margin (EggPageSettings    *settings)
{
  return settings->right_margin;
}

void
egg_page_settings_set_right_margin (EggPageSettings    *settings,
				    double              margin)
{
  settings->right_margin = margin;
}

/* These take orientation, but not margins into consideration */
double
egg_page_settings_get_paper_width (EggPageSettings    *settings)
{
  if (settings->orientation == EGG_PAGE_ORIENTATION_PORTRAIT) {
    return egg_paper_size_get_width (settings->paper_size);
  } else {
    return egg_paper_size_get_height (settings->paper_size);
  }
}

double
egg_page_settings_get_paper_height (EggPageSettings    *settings)
{
  if (settings->orientation == EGG_PAGE_ORIENTATION_PORTRAIT) {
    return egg_paper_size_get_height (settings->paper_size);
  } else {
    return egg_paper_size_get_width (settings->paper_size);
  }
}

/* These take orientation, and margins into consideration */
double
egg_page_settings_get_page_width (EggPageSettings    *settings)
{
  double width;
  
  width = egg_page_settings_get_paper_width (settings);
  width -= settings->left_margin + settings->right_margin;
  
  return width;
}

double
egg_page_settings_get_page_height (EggPageSettings    *settings)
{
  double height;
  
  height = egg_page_settings_get_paper_height (settings);
  height -= settings->top_margin + settings->bottom_margin;
  
  return height;
}
