/* GTK - The GIMP Toolkit
 * eggprintoperation.c: Print Operation
 * Copyright (C) 2005, Red Hat, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "eggprintoperation.h"
#include "eggprintmarshal.h"
#include <cairo-pdf.h>

#define EGG_PRINT_OPERATION_GET_PRIVATE(obj)(G_TYPE_INSTANCE_GET_PRIVATE ((obj), EGG_TYPE_PRINT_OPERATION, EggPrintOperationPrivate))

#define MM_PER_INCH 25.4
#define POINTS_PER_INCH 72

enum {
  BEGIN_PRINT,
  REQUEST_PAGE_SETTINGS,
  DRAW_PAGE,
  END_PRINT,
  LAST_SIGNAL
};

struct _EggPrintOperationPrivate
{
  EggPageSettings *default_page_settings;
  EggPrinterSettings *printer_settings;
  int nr_of_pages;
  gboolean use_full_page;
  gboolean show_dialog;
  char *pdf_target;

  /* Data for the print job: */
  cairo_surface_t *surface;
  double units_per_mm_x, units_per_mm_y;
};


static guint signals[LAST_SIGNAL] = { 0 };

G_DEFINE_TYPE (EggPrintOperation, egg_print_operation, G_TYPE_OBJECT)

static void
egg_print_operation_finalize (GObject *object)
{
  EggPrintOperation *print_operation = EGG_PRINT_OPERATION (object);

  g_object_unref (print_operation->priv->default_page_settings);
  g_object_unref (print_operation->priv->printer_settings);
  g_free (print_operation->priv->pdf_target);

  G_OBJECT_CLASS (egg_print_operation_parent_class)->finalize (object);
}

static void
egg_print_operation_init (EggPrintOperation *operation)
{
  operation->priv = EGG_PRINT_OPERATION_GET_PRIVATE (operation);

  operation->priv->default_page_settings = egg_page_settings_new ();
  operation->priv->printer_settings = egg_printer_settings_new ();
  operation->priv->nr_of_pages = EGG_PRINT_UNKNOWN_NR_OF_PAGES;
  operation->priv->use_full_page = FALSE;
  operation->priv->show_dialog = TRUE;
  operation->priv->pdf_target = NULL;
}

static void
egg_print_operation_class_init (EggPrintOperationClass *class)
{
  GObjectClass *gobject_class = (GObjectClass *)class;

  gobject_class->finalize = egg_print_operation_finalize;
  
  g_type_class_add_private (gobject_class, sizeof (EggPrintOperationPrivate));


  signals[BEGIN_PRINT] =
    g_signal_new ("begin_print",
		  G_TYPE_FROM_CLASS (gobject_class),
		  G_SIGNAL_RUN_LAST,
		  G_STRUCT_OFFSET (EggPrintOperationClass, begin_print),
		  NULL, NULL,
		  g_cclosure_marshal_VOID__VOID,
		  G_TYPE_NONE, 0);
  signals[REQUEST_PAGE_SETTINGS] =
    g_signal_new ("request_page_settings",
		  G_TYPE_FROM_CLASS (gobject_class),
		  G_SIGNAL_RUN_LAST,
		  G_STRUCT_OFFSET (EggPrintOperationClass, request_page_settings),
		  NULL, NULL,
		  _egg_marshal_VOID__INT_OBJECT,
		  G_TYPE_NONE, 2,
		  G_TYPE_INT,
		  EGG_TYPE_PAGE_SETTINGS);
  signals[DRAW_PAGE] =
    g_signal_new ("draw_page",
		  G_TYPE_FROM_CLASS (gobject_class),
		  G_SIGNAL_RUN_LAST,
		  G_STRUCT_OFFSET (EggPrintOperationClass, draw_page),
		  NULL, NULL,
		  _egg_marshal_BOOL__POINTER_INT_OBJECT,
		  G_TYPE_BOOLEAN, 3,
		  G_TYPE_POINTER,
		  G_TYPE_INT,
		  EGG_TYPE_PAGE_SETTINGS);
  signals[END_PRINT] =
    g_signal_new ("end_print",
		  G_TYPE_FROM_CLASS (gobject_class),
		  G_SIGNAL_RUN_LAST,
		  G_STRUCT_OFFSET (EggPrintOperationClass, end_print),
		  NULL, NULL,
		  g_cclosure_marshal_VOID__VOID,
		  G_TYPE_NONE, 0);
}

EggPrintOperation *
egg_print_operation_new ()
{
  EggPrintOperation *print_operation;

  print_operation = g_object_new (EGG_TYPE_PRINT_OPERATION, NULL);
  
  return print_operation;
}

void
egg_print_operation_set_default_page_settings (EggPrintOperation  *op,
					       EggPageSettings    *default_page_settings)
{
  g_return_if_fail (op != NULL);
  g_return_if_fail (default_page_settings != NULL);
  
  if (default_page_settings != op->priv->default_page_settings)
    {
      g_object_unref (op->priv->default_page_settings);
      op->priv->default_page_settings = g_object_ref (default_page_settings);
    }
}

EggPageSettings *
egg_print_operation_get_default_page_settings (EggPrintOperation  *op)
{
  g_return_val_if_fail (op != NULL, NULL);

  return op->priv->default_page_settings;
}

void
egg_print_operation_set_printer_settings (EggPrintOperation  *op,
					  EggPrinterSettings *printer_settings)
{
  g_return_if_fail (op != NULL);
  g_return_if_fail (printer_settings != NULL);
  
  if (printer_settings != op->priv->printer_settings)
    {
      g_object_unref (op->priv->printer_settings);
      op->priv->printer_settings = g_object_ref (printer_settings);
    }
}

EggPrinterSettings *
egg_print_operation_get_printer_settings (EggPrintOperation  *op)
{
  g_return_val_if_fail (op != NULL, NULL);

  return op->priv->printer_settings;
}

void
egg_print_operation_set_nr_of_pages (EggPrintOperation  *op,
				     int                 n_pages)
{
  g_return_if_fail (op != NULL);

  op->priv->nr_of_pages = n_pages;
}

void
egg_print_operation_set_use_full_page (EggPrintOperation  *op,
				       gboolean            full_page)
{
  g_return_if_fail (op != NULL);

  op->priv->use_full_page = full_page;
}

void
egg_print_operation_set_show_dialog (EggPrintOperation  *op,
				     gboolean            show_dialog)
{
  g_return_if_fail (op != NULL);

  op->priv->show_dialog = show_dialog;
}

void
egg_print_operation_set_pdf_target (EggPrintOperation  *op,
				    const char *        filename)
{
  g_return_if_fail (op != NULL);

  op->priv->pdf_target = g_strdup (filename);
}

static double
mm_to_points (double mm)
{
  return mm * POINTS_PER_INCH/MM_PER_INCH;
}

static gboolean
run_print_dialog (EggPrintOperation  *op)
{
  if (op->priv->pdf_target != NULL) {
    double width, height;
    /* This will be overwritten later by the non-default size, but
       we need to pass some size: */
    width = egg_page_settings_get_paper_width (op->priv->default_page_settings);
    width = mm_to_points (width);
    height = egg_page_settings_get_paper_height (op->priv->default_page_settings);
    height = mm_to_points (height);
    
    op->priv->surface = cairo_pdf_surface_create (op->priv->pdf_target,
						  width, height);
    /* TODO: Set DPI? From settings object? */

    op->priv->units_per_mm_x = POINTS_PER_INCH/MM_PER_INCH;
    op->priv->units_per_mm_y = POINTS_PER_INCH/MM_PER_INCH;
    
    return TRUE; /* No dialog for pdf writing */
  }


  /* Open print dialog */
  /* set print settings on dialog */
  /* run dialog, if show_dialog */
  /* extract print settings from dialog */
  /* if print selected, return TRUE, else FALSE */

  return TRUE;
}

void
egg_print_operation_run (EggPrintOperation  *op)
{
  cairo_t *cr;
  int page;
  EggPageSettings *page_settings;
  gboolean draw_page_res;
  
  if (!run_print_dialog (op))
    return;
  
  cr = cairo_create (op->priv->surface);
  cairo_scale (cr,
	       op->priv->units_per_mm_x,
	       op->priv->units_per_mm_y);
  

  g_signal_emit (op, signals[BEGIN_PRINT], 0);

  page = 0;
  while (op->priv->nr_of_pages == EGG_PRINT_UNKNOWN_NR_OF_PAGES ||
	 page < op->priv->nr_of_pages)
    {
      page_settings = egg_page_settings_copy (op->priv->default_page_settings);

      g_signal_emit (op, signals[REQUEST_PAGE_SETTINGS], 0, page, page_settings);

      /* TODO: set cairo_t page settings from page_settings, needs new API */

      cairo_save (cr);

      if (!op->priv->use_full_page)
	{
	  cairo_translate (cr,
			   egg_page_settings_get_left_margin (page_settings),
			   egg_page_settings_get_top_margin (page_settings));
	}
      
      g_signal_emit (op, signals[DRAW_PAGE], 0, 
		     cr, page, page_settings,
		     &draw_page_res);

      if (!draw_page_res && op->priv->nr_of_pages == EGG_PRINT_UNKNOWN_NR_OF_PAGES)
	op->priv->nr_of_pages = page;

      if (draw_page_res)
	{
	  cairo_show_page (cr);
	}
      
      cairo_restore (cr);

      g_object_unref (page_settings);
      

      page++;
    }
  
  g_signal_emit (op, signals[END_PRINT], 0);


  cairo_destroy (cr);
  cr = NULL;

  cairo_surface_destroy (op->priv->surface);
  op->priv->surface = NULL;
}
